//-------------------------------
//   5530 - Defines and Typedefs
//-------------------------------

#ifndef CX5530_H
#define CX5530_H

//
//  waveform audio error return values 
//
#define WAVERR_BADFORMAT      (32)  // unsupported wave format
#define WAVERR_STILLPLAYING   (33)  // still something playing
#define WAVERR_UNPREPARED     (34)  // header not prepared
#define WAVERR_SYNC           (35)  // device is synchronous
#define WAVERR_LASTERROR      (35)  // last error in range

//
//  General error return values 
//
#define MMSYSERR_NOERROR      0     // no error 
#define MMSYSERR_ALLOCATED    (4)   // device already allocated 
#define MMSYSERR_INVALHANDLE  (5)   // device handle is invalid 

typedef unsigned short    MMRESULT;                                   

//
//    PCM Format value
//
//    The caller has to specify the format of the wave data.
//    At this point, the driver does only handles PCM data.
// 
#define WAVE_FORMAT_PCM         1   

//
//  Silence to fill the audio buffers
//
#define AUDIO_SILENCE           0

//
//  PCI Config address. 
//  Bits 15..11 specify the PCI device.
//  Bits 7..2 specify reg in PCI device.
//
#define PCI_CADDR         0x0CF8  

//                              
//  PCI Config data reg.
//  After PCI_ADDR set, this reg contains    
//  the corresponding data (the dev reg).
//
#define PCI_CDATA         0x0CFC  

//
//  Device and Vendor IDs
//

//  Following is the PCI Bridge (Func 0) ID.                                      
//  CYRIX's PCI Vendor ID.                                           
//
#define CYRIX_VENDOR_ID     0x1078   
#define NATIONAL_VENDOR_ID  0x100B   

//
//       Audio  Device IDs
//
#define	CX5530_DEV_ID    0x0103
#define	SC1200_DEV_ID    0x0503

//
//  This driver works with Scorpius and Unicorn Platforms
//
#define UNICORN_BRIDGE    (CYRIX_VENDOR_ID+0x1000000)  // 5530+Marmot   PCI code).
#define SCORPIUS_BRIDGE   (CYRIX_VENDOR_ID+0x4000000)  // 5530+Scorpius PCI ID).
#define CARMEL_BRIDGE     (NATIONAL_VENDOR_ID+0x5000000)  // 5540+Carmel PCI ID).

#define UNICORN_BRIDGE_0100   0x0100  // Unicorn PCI ID).

//
// Function 3 of 5530 PCI dev is Audio (ISA idx).
//
#define  PCI_FUNC3_AUDIO     0x300       
#define  PCI_AUDIO_CMD_REG   0x04

//
// Byte offset in mem mapped audio regs.    
//
#define  CODEC_CMD_REG       0x0c                                 

//
//  Return value for AudioGetInterruptType()
//
//  AudioGetInterruptType will return to the caller 
//  one of the following states:
//

#define AUDIO_STATE_IGNORE          0
#define AUDIO_STATE_IN_RECORDING    0x01
#define AUDIO_STATE_IN_OVERFLOW     0x02
#define AUDIO_STATE_IN_STOPPED      0x03
#define AUDIO_STATE_IN_MASK         0x0F
#define AUDIO_STATE_OUT_PLAYING     0x10
#define AUDIO_STATE_OUT_UNDERFLOW   0x20
#define AUDIO_STATE_OUT_STOPPED     0x30
#define AUDIO_STATE_OUT_MASK        0xF0

typedef unsigned char AUDIO_STATE;

#define WPDM_CLOSE          1
#define WPDM_CONTINUE       2
#define WPDM_ENDOFDATA      3
#define WPDM_GETDEVCAPS     4
#define WPDM_GETVOLUME      5
#define WPDM_OPEN           6
#define WPDM_PAUSE          7
#define WPDM_RESTART        8
#define WPDM_SETVOLUME      9
#define WPDM_STANDBY        10
#define WPDM_START          11
#define WPDM_STOP           12


//
//   Bit conversions
//
#define BITS_8_TO_16(x) ( ( (long) ((unsigned char) x - 128) ) << 8 )
#define BITS_16_TO_8(x) ( ( (unsigned char) ((long) x >> 8 ) ) + 128 )

//
// The CODEC commands are actually 16-bit words, into which is inserted
// the codec "target" register, identified by a byte. The 5530 Codec
// controller writes a command unsigned short of 32-bits, that includes the codec
// command unsigned short.
//
#define KAHLUA_CODEC_COMMAND_MASK       0xFF00FFFF
#define KAHLUA_CODEC_NUM                ( 0 << 22)
#define RECORD_DMA_BUFFER_BASE (AUDIO_DMA_BUFFER_BASE + 2 * AUDIO_DMA_PAGE_SIZE)

//
//  The Interaction with the CODEC is a bit cumbersome 
//  because of the serial interface.
//
#define CODEC_STATUS_REG    0x08        // In Audio mem-map.
#define CODEC_CMD_REG       0x0c        // In audio mem-map.
#define CODEC_CMD_VALID     0x010000
#define CODEC_STATUS_VALID  0x00030000  // Bits 17..16. Status valid and new.

//
// Codec register indexes. Note these are all shifted left by 16 bits.
//
#define LM4548_MASTER_VOLUME        0x0200
#define LM4548_VIDEO_VOLUME         0x1400
#define LM4548_TV_VOLUME            0x1600
#define LM4548_PCM_OUT_VOL          0x1800
#define LM4548_EXT_AUDIO_CTRL_STAT  0x2A00
#define LM4548_PCM_FRONT_DAC_RATE   0x2C00
#define LM4548_PCM_LR_ADC_RATE      0x3200

//
//   Physical address of the device
//
#define AUDIO_REGS_BUFFER           0x40011000  

//  
//    Typedefs
//
typedef enum 
{
	WAPI_IN = 0, 
	WAPI_OUT
} WAPI_INOUT, *PWAPI_INOUT;

typedef enum 
{
	PCM_TYPE_M8,
	PCM_TYPE_M16,
	PCM_TYPE_S8,
	PCM_TYPE_S16
} PCM_TYPE, *PPCM_TYPE;

typedef struct  
{
	unsigned char sample;              // Unsigned 8-bit sample
} SAMPLE_8_MONO;

typedef struct  
{
	signed short sample;               // Signed 16-bit sample
} SAMPLE_16_MONO;

typedef struct  
{
	unsigned char sample_left;         // Unsigned 8-bit sample
	unsigned char sample_right;        // Unsigned 8-bit sample
} SAMPLE_8_STEREO;

typedef struct  
{
	unsigned short sample_left;        // Signed 16-bit sample
	unsigned short sample_right;       // Signed 16-bit sample
} SAMPLE_16_STEREO;

typedef union 
{
	SAMPLE_8_MONO m8;
	SAMPLE_16_MONO m16;
	SAMPLE_8_STEREO s8;
	SAMPLE_16_STEREO s16;
} PCM_SAMPLE, *PPCM_SAMPLE;

//
//   Extended waveform format structure used for all non-PCM formats. this
//   structure is common to all non-PCM formats.
//
typedef struct tWAVEFORMATEX
{
	unsigned short  wFormatTag;         // format type 
	unsigned short  nChannels;          // number of channels (i.e. mono, stereo...) 
	unsigned long   nSamplesPerSec;     // sample rate 
	unsigned long   nAvgBytesPerSec;    // for buffer estimation 
	unsigned short  nBlockAlign;        // block size of data 
	unsigned short  wBitsPerSample;     // number of bits per sample of mono data 
	unsigned short  cbSize;             // the count in bytes of the size of 
                                        // extra information (after cbSize) 
} WAVEFORMATEX, *PWAVEFORMATEX,  *NPWAVEFORMATEX,  *LPWAVEFORMATEX;
typedef const WAVEFORMATEX  *LPCWAVEFORMATEX;

WAPI_INOUT  GlobalApidir;

//
//  Wave data block header 
//
typedef struct wavehdr_tag 
{
	char *              lpData;                 // pointer to locked data buffer 
	unsigned long       dwBufferLength;         // length of data buffer 
	unsigned long       dwBytesRecorded;        // used for input only 
	unsigned long		 dwBytesRead;        // Number of bytes Read in this buffer
	unsigned long       dwUser;                 // for client's use 
	unsigned long       dwFlags;                // assorted flags (see defines) 
	unsigned long       dwLoops;                // loop control counter 
	struct wavehdr_tag *lpNext;                 // reserved for driver 
	unsigned long       reserved;               // reserved for driver 
} WAVEHDR, *PWAVEHDR, *NPWAVEHDR, *LPWAVEHDR;

//
//   Common stuff
//
#define     TRUE    1
#define     FALSE   0
#define     NULL    ((void *)0)

#endif // CX5530_H

